@@ -9,6 +9,7 @@
 #
 from argparse import ArgumentParser, Namespace
 from logging import DEBUG, INFO
+from pathlib import Path
 from traceback import format_exc as get_traceback
 from typing import Final, Optional, Protocol, cast, final
 
@@ -23,11 +24,15 @@ from .action import (
     create_actions,
 )
 from .config import Config, ConfigHolder
+from .dynamic import DynamicVersionSource
+from .source import StaticPep621VersionSource
 from .logging import TRACE, logger, traced_function
 from .version import Pep440VersionFormatter, Version
 
 
 class _ProjectLike(ConfigHolder, Protocol):
+    root: Path
+
     @property
     def pyproject_file(self) -> str:
         # Method empty: Only a protocol stub
@@ -37,6 +42,22 @@ class _ProjectLike(ConfigHolder, Protocol):
         # Method empty: Only a protocol stub
         pass
 
+class _VersionSource(Protocol):
+    @property
+    def is_enabled(self) -> bool:
+        raise NotImplementedError()
+
+    def __get_current_version(self) -> Version:
+        raise NotImplementedError()
+
+    def __set_current_version(self, v: Version) -> None:
+        raise NotImplementedError()
+
+    current_version = property(__get_current_version, __set_current_version)
+
+    def save_value(self) -> None:
+        raise NotImplementedError()
+
 
 @final
 class BumpCommand(BaseCommand):
@@ -102,15 +123,20 @@ class BumpCommand(BaseCommand):
         config: Config = Config(project)
         self._setup_logger(options.trace, options.debug)
 
-        version_value: Optional[str] = cast(
-            Optional[str], config.get_pyproject_value("project", "version")
-        )
+        static_backend: _VersionSource = StaticPep621VersionSource(project, config)
+        dynamic_backend: _VersionSource = DynamicVersionSource(project.root, config)
 
-        if version_value is None:
+        selected_backend: Optional[_VersionSource] = None
+        for backend in (static_backend, dynamic_backend):
+            if backend.is_enabled:
+                selected_backend = backend
+                break
+
+        if selected_backend is None:
             logger.error("Cannot find version in %s", project.pyproject_file)
             return
 
-        version: Version = Version.from_string(version_value)
+        version: Version = selected_backend.current_version
 
         actions: ActionCollection = self._get_actions(
             options.micro, options.reset, options.remove
@@ -130,11 +156,14 @@ class BumpCommand(BaseCommand):
             logger.debug("Exception occurred: %s", get_traceback())
             raise SystemExit(1) from exc
 
-        next_version: str = self._version_to_string(result)
+        backend.current_version = result
 
-        config.set_pyproject_value(next_version, "project", "version")
+        current_version: str = Pep440VersionFormatter().format(version)
+        next_version: str = Pep440VersionFormatter().format(result)
         if not options.dry_run:
-            project.write_pyproject(True)
+
+            logger.info("Updating version: %s -> %s", current_version, next_version)
+            backend.save_value()
         else:
             logger.info("Would write new version %s", next_version)
 
